
JavaInStObject subclass: #JavaVMExample
  instanceVariableNames: 
    ' jObject notifier '
  classVariableNames: ''
  poolDictionaries: ''    !


!JavaVMExample class methods !
   
nativeExamples: example
	"Activate Smalltalk native examples."

	| essay |
	essay := JNIClass named: self nativeExamplesClassName.
	example jObject: essay newObject.
	essay registerNatives: (JavaNativeMethods
		fromArray: self nativeExamplesSpecification
		withReceiver: example).
	example runNatives.
	essay unRegisterNatives.!
  
nativeExamplesClassName
	"Returns the java class name for reflexion essays."

	^'Examples.NativeExamples'!
   
nativeExamplesSpecification
	"Returns the native examples specifications."

	^#(

		"public native byte nativeByte() ;"
		(	'nativeByte' ((returns: byte))
			nil nativeByte:receiver:
			(ulong ulong) ulong
		)

		"public native void nativeVoid() ;"
		(	'nativeVoid' ()
			nil nativeVoid:receiver:
			(ulong ulong) none
		)

		"public native int nativeThisInt( int theInt) ;"
		(	'nativeThisInt' (int (returns: int))
			nil nativeThisInt:receiver:int:
			(ulong ulong ulong) ulong
		)

		"public native boolean nativeConfirm( String text) ;"
		(	'nativeConfirm' (string (returns: boolean))
			nil nativeConfirm:receiver:label:
			(ulong ulong ulong) ulong
		)

	)!

numericExamplesClassName
	"Returns the java class name for this examples."

	^'Examples.NumericExamples'!

run
	"Run all examples in order"

	| example support |
	example := self new.

	example
		booleanTests;
		stringTests;
		numericTests;
		reflexionTests;
		yourself.

	"Native Callbacks to Smalltalk examples."
	support := JNIClass
		named: self nativeExamplesClassName
		ifAbsent: [].
	support isNil
	ifTrue: [
		example
			notify: 'Class for native examples not found.';
			notify: 'Native examples *skipped*'.
	]
	ifFalse: [ self nativeExamples: example ].
!
  
stringExamplesClassName
	"Returns the java class name for this examples."

	^'Examples.StringExamples'! !



!JavaVMExample methods !

booleanTests
	"Boolean access tests."

	| boolean jTrue jFalse |
	self notify: 'Boolean Tests -=-=-=-=-=-=-=-=-=-=-=-=-'.

	boolean := JNIClass boolean.
	self notify: 'Boolean class = ',boolean asString.

	jTrue := boolean true.
	self notify: 'true = ',jTrue asString.
	jFalse := boolean false.
	self notify: 'false = ',jFalse asString.

	jTrue isTrue ifFalse: [
		self notify: 'jTrue is not True.  :-('.
		self notify: 'jTrue isInstanceOf: java.lang.Boolean = '
			,(jTrue isInstanceOf: JNIClass boolean) asString.
	].
	jFalse isFalse ifFalse: [
		self notify: 'jFalse is not False.  :-('.
		self notify: 'jFalse isInstanceOf: java.lang.Boolean = '
			,(jFalse isInstanceOf: JNIClass boolean) asString.
	].!
 
cr
	"Write a cr in the notifier."

	self notifier cr!

initializeNotifier
	" Private - Initializes the receiver's notifier"

	notifier := TextWindow
		windowLabeled: ' JavaVMExample results. '
		frame: (0@0 extent: 200@300)!
  
jObject
    "Returns the instance var of the receiver
     for external manipulation"
    ^jObject!
  
jObject: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    jObject := anObject!
 
nativeByte: environment receiver: receiver
	"Serves the native callback method."

	MessageBox message:
		'nativeByte: ', environment asString,
		'\receiver: 'withCrs, (JNIObject value: receiver from: nil) asString,
		'\Returning zero...'withCrs.
	^77!
  
nativeConfirm: environment receiver: receiver label: text
	"Serves the native callback method."

	| result |
	result := MessageBox confirm:
		'nativeVoid: ', environment asString,
		'\receiver: 'withCrs, (JNIObject value: receiver from: nil) asString,
		'\'withCrs, (JNIString value: text from: nil) asString.
	^result!
 
nativeThisInt: environment receiver: receiver int: anInteger
	"Serves the native callback method."

	MessageBox message:
		'nativeVoid: ', environment asString,
		'\receiver: 'withCrs, (JNIObject value: receiver from: nil) asString,
		' int: ', anInteger asString.
	^anInteger!

nativeVoid: environment receiver: receiver
	"Serves the native callback method."

	MessageBox message:
		'nativeVoid: ', environment asString,
		'\receiver: 'withCrs, (JNIObject value: receiver from: nil) asString,
		'\void return...'withCrs.
	^nil!

notifier
    "Returns the instance var of the receiver
     for external manipulation"

	notifier isNil ifTrue: [ self initializeNotifier ].
    ^notifier!

notifier: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    notifier := anObject!
   
notify: aString
	" Notify aString onto receiver's notifier."

    self notifier cr;show: aString!

numericTests
	"Numeric access tests."

	| aClass example anObject
	  method name returnType methodClass result
	|

	self cr;notify: 'Numeric Tests -=-=-=-=-=-=-=-=-=-=-=-=-'.
	aClass := JNIClass named: self class numericExamplesClassName.
	example := aClass newObject.

	self notify: 'example = ',example asString.
	self notify: 'anObject = ',(example invoke: 'getAnObject')asString.

	self notify: 'now settings example.anObject with this (recursive containment)'.
	example invoke: 'setAnObject' with: example.	
	self notify: 'anObject = ',(example invoke: 'getAnObject')asString.
	self notify: 'Is anObject = example ? >> ',(
				example
					invoke: 'equals'
					with: (example invoke: 'getAnObject')
			) isTrue asString.

	self notify: 'Numeric type checkings.......................'.

	self notify: 'Numeric Getters...'.
	#("(selector returnType)"
		" (getAnObject object) "
		(getByte0 byte) (getByte1 byte) (getByte100 byte)
		(getByteMinus1 byte) (getByteMinus77 byte)
		(getCharA char)
		 "
		(getDouble0 double) (getDouble1 double) (getDouble10 double)
		(getDoubleMinus1 double) (getDoubleMinus77 double)
		"
		(getFalse boolean)
		(getFloat0 float) (getFloat10 float) (getFloatMinus77 float)
		(getInt0 int) (getInt1 int) (getIntMinus1 int) (getIntMinus77 int)
		(getLong0 long) (getLong1 long) (getLong77 long) (getLongMinus77 long)
		" (getNull object) "
		(getShort0 short) (getShort100 short) (getShortMinus1 short) (getShortMinus77 short)
		(getTrue boolean)
		(getVoid void)
	) do: [:tuple|
		name := tuple first.
		returnType := tuple last.
		"get method class"
		methodClass := JNIMethod virtual perform: returnType.
		method := methodClass
		    of: example jClass named: name asString
		    signature: (JavaMethodSignature returning: returnType).
		[ result := method evaluate: example ]
		on: Error
		do: [:ex| result := '*Error* ',ex asString ].
		self notify: '#',name,' returns ', result asString.
	].

"currently does not work with invoke... :-(
	self notify: '3 + 4 = ', (example invoke: 'getIntSum' with: 3 with: 4)asString.
"
	method := JNIMethod virtual int
		of: example jClass named: 'getIntSum'
	    signature: (JavaMethodSignature new int int returns: #int).
	self notify: '3 + 4 = ',
		(method evaluate: example withArguments:#(3 4)) asString.



	self notify: 'done......................................................'.!
   
reflexionTests
	"Reflexion access tests."

	| aClass example anObject methods |
	self cr;notify: 'Reflexion Tests -=-=-=-=-=-=-=-=-=-=-=-=-'.
	aClass := JNIClass named: self class numericExamplesClassName.
	self notify: 'Exploring ',aClass asString.
	methods := aClass getMethods.
	self notify: 'All (implemented+inherited) Methods '.
	methods do: [:each|
		self notify: '    ',each toString asString
	].
	methods := aClass getConstructors.
	self notify: 'Constructors'.
	methods do: [:each|
		self notify: '    ',each toString asString
	].
	methods := aClass getFields.
	self notify: 'All (defined+inherited) Fields'.
	methods do: [:each|
		self notify: '    ',each toString asString
	].!
  
runNatives
	" Private - Run Native examples.
	See method JavaVMExample class>>nativeExamples."

	self cr;notify: 'Callbacks Tests -=-=-=-=-=-=-=-=-=-=-=-=-'.
	self notify: 'Start running natives...'.

	self notify: 'nativeByte()->', (jObject invoke: 'nativeByte') asString.
	self notify: 'nativeVoid->', (jObject invoke: 'nativeVoid') asString.
"	self notify: 'nativeThisInt->', (jObject invoke: 'nativeThisInt' with: 77) asString. "
	self notify: 'nativeConfirm->', (jObject invoke: 'nativeConfirm'
		with: (JNIString fromString: 'Do you like this example ?')) asString.

	self notify: 'End running natives...'.!
  
stringTests
	"String access tests."

	| string example hello |
	self cr;notify: 'String Tests -=-=-=-=-=-=-=-=-=-=-=-=-'.
	string := JNIClass string.
	self notify: 'String class = ',string asString.

	example := (JNIClass named:
		self class stringExamplesClassName)
		newObject.

	self notify: 'Default hello = ',(example invoke: 'getHello') asString.
	hello := (JNIString from: 'New Hello String. Today is ', Date today asString).
	example invoke: 'setHello' with: hello.	
	self notify: 'New hello value = ',(example invoke: 'getHello') asString.
	self notify: 'Is hello ? = ',(example invoke: 'isHello' with: hello) asString.
! !
